package org.jboss.as.integration.hornetq.deployers;

import java.util.concurrent.atomic.AtomicInteger;

import org.hornetq.core.config.Configuration;
import org.jboss.as.integration.hornetq.deployers.pojo.HornetQCoreDeployment;
import org.jboss.beans.metadata.plugins.builder.BeanMetaDataBuilderFactory;
import org.jboss.beans.metadata.spi.BeanMetaData;
import org.jboss.beans.metadata.spi.builder.BeanMetaDataBuilder;
import org.jboss.deployers.spi.DeploymentException;
import org.jboss.deployers.spi.deployer.helpers.AbstractSimpleRealDeployer;
import org.jboss.deployers.structure.spi.DeploymentUnit;
import org.jboss.metadata.ejb.jboss.JBossMetaData;

/**
 * In a regular flow HornetQCoreParserDeployer will parer hornetq-configuration.xml or hornetq-queues.xml and it
 * will jump-start the deployment for Configuration.class. This deployer will
 * then be called directly by the VFS Deployment Framework, as the Input object
 * for this is a Configuration class.
 * 
 * In an alternate flow, other deployers (e.g. TorqueBox) may install a Configuration object
 * directly and this deployer will take over without the need of a real XML file
 * 
 * @author <mailto:clebert.suconic@jboss.org">Clebert Suconic</a>
 * 
 */
public class HornetQCoreConfigRealDeployer extends AbstractSimpleRealDeployer<Configuration>
{

    private final AtomicInteger idGenerator = new AtomicInteger(1);

    public HornetQCoreConfigRealDeployer()
    {
        super(Configuration.class);
        addOutput(BeanMetaData.class);
        
        // This is just to guarantee the order. JBossMetaData deployment should happen before
        addInput(JBossMetaData.class);
    }

    @Override
    public void deploy(DeploymentUnit unit, Configuration config) throws DeploymentException
    {
       if (log.isTraceEnabled())
       {
          log.trace("Deploying unit: " + unit + " with config " + config);
       }
        String name = "HornetQCoreDeployment::" + idGenerator.incrementAndGet();
        
        BeanMetaDataBuilder builder = BeanMetaDataBuilderFactory.createBuilder(name, HornetQCoreDeployment.class.getName());

        builder.addPropertyMetaData("name", name);

        builder.addPropertyMetaData("server", builder.createInject("HornetQServer"));
        
        builder.addPropertyMetaData("config", config);

        BeanMetaData bean = builder.getBeanMetaData();

        unit.addAttachment(name, bean);
    }

}
